; ***********************************************************
; Interrupcion 0: comprueba fin de partida
; ***********************************************************

Interrupcion0:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion1
      ld (#39), BC
      
      ; ld A, COLOR_BRIGHT_YELLOW
      ; call ColorBorde

      ; Incrementamos el contador de frames
      ld A, (CONTADOR_FRAMES)
      inc A
      ld (CONTADOR_FRAMES), A

      ; Solo actualizamos cada dos frames (25Hz)
      ld A, (CONTADOR_FRAMES)
      bit 0, A
      jp Z, Interrupcion1_Fin         
      
      ; Fin de la partida si no quedan huecos en pantalla
      call PantallaCompletada ; A = 1 si pantalla completada
      cp 1
      jr NZ, Interrupcion0_CompruebaBloqueo

      ; Fin de partida
      ld A, COLOR_BRIGHT_WHITE
      call ColorBorde     

      ; Flag GAME OVER = 2
      ld A, 2
      ld (GAME_OVER), A

      jr Interrupcion0_Fin

Interrupcion0_CompruebaBloqueo:
      ; Si despues del ultimo movimiento estamos bloqueados,
      ; perdemos y reiniciamos la pantalla
      call Bloqueado          ; A = 1 si no hay movimientos posibles
      cp 1
      jr NZ, Interrupcion0_Fin

      ; Fin de partida
      ld A, COLOR_BRIGHT_RED
      call ColorBorde      

      ; Flag GAME OVER = 1
      ld A, 1
      ld (GAME_OVER), A

      
Interrupcion0_Fin:

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti
      
; ***********************************************************
; Interrupcion 1: lectura de teclado
; ***********************************************************

Interrupcion1:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion2
      ld (#39), BC

      ; ld A, COLOR_BRIGHT_RED
      ; call ColorBorde

      ; Solo actualizamos cada dos frames (25Hz)
      ld A, (CONTADOR_FRAMES)
      bit 0, A
      jr Z, Interrupcion1_Fin      

      ; Si el gato se esta moviendo, ignoramos el teclado
      ld A, (GATO_MUEVE_NEXT)
      cp DIR_STOP
      jr NZ, Interrupcion1_Fin

      ; Comprueba teclas de direccion
      call LeerTeclado
      ld HL, TECLAS_PULSADAS
      ld D, (HL)

      ; Si sigue pulsada la misma tecla, ignorarla
      ; para evitar que en el cambio de pantallas
      ; siga pulsada y se mueva el gato.
      ld A, (TECLA_PULSADA)
      cp 0
      jr Z, Interrupcion1_CompruebaTeclas ; No habia tecla pulsada antes
      cp D
      jr Z, Interrupcion1_Fin ; Sigue pulsada la misma tecla, fin

Interrupcion1_CompruebaTeclas:
      ; Derecha?
      bit KEY_RIGHT, D
      jr Z, Interrupcion1_CompruebaIzquierda
      ld A, DIR_RIGHT
      ld (GATO_MUEVE_NEXT), A
      jr Interrupcion1_GuardaTecla

Interrupcion1_CompruebaIzquierda:
      ; Izquierda?
      bit KEY_LEFT, D
      jr Z, Interrupcion1_CompruebaArriba
      ld A, DIR_LEFT
      ld (GATO_MUEVE_NEXT), A
      jr Interrupcion1_GuardaTecla

Interrupcion1_CompruebaArriba:
      ; Arriba?
      bit KEY_UP, D
      jr Z, Interrupcion1_CompruebaAbajo
      ld A, DIR_UP
      ld (GATO_MUEVE_NEXT), A
      jr Interrupcion1_GuardaTecla

Interrupcion1_CompruebaAbajo:
      ; Abajo?
      bit KEY_DOWN, D
      jr Z, Interrupcion1_CompruebaEscape
      ld A, DIR_DOWN
      ld (GATO_MUEVE_NEXT), A
      jr Interrupcion1_GuardaTecla

Interrupcion1_CompruebaEscape:
      ; ESC?
      bit KEY_ESCAPE, D
      jr Z, Interrupcion1_GuardaTecla
      pop AF      ; Restauramos la pila y confirmamos salida
      pop BC
      pop DE
      pop HL      
      jp ReinciarJuego

Interrupcion1_GuardaTecla:

      ; Guardar la tecla pulsada antes de salir
      ld A, D
      ld (TECLA_PULSADA), A

Interrupcion1_Fin:      

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti

; ***********************************************************
; Interrupcion 2: mover el gato
; ***********************************************************

Interrupcion2:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion3
      ld (#39), BC
      
      ; ld A, COLOR_BRIGHT_WHITE
      ; call ColorBorde

      ; Solo actualizamos cada dos frames (25Hz)
      ld A, (CONTADOR_FRAMES)
      bit 0, A
      jp Z, Interrupcion2_Fin          

      ; Actualizamos las coordenadas del gato si esta moviendose

      ld A, (GATO_MUEVE_NEXT)
      cp DIR_STOP       ; Gato parado?
      jr Z, Interrupcion2_Fin
      cp DIR_RIGHT      ; Moviendose a la derecha?
      jr NZ, Interrupcion2_MueveIzquierda
      ; Comprueba si el gato puede moverse hacia la derecha
      ld A, (GATO_X)
      inc A
      cp MAX_COLUMNAS
      jr NC, Interrupcion2_Stop
      ld D, A
      ld A, (GATO_Y)
      ld E, A
      call BloqueXY           ; En A el tipo de bloque en (D,E)
      cp 1                    ; Bloque solido, no puede avanzar
      jr Z, Interrupcion2_Stop      ; Paramos al gato
      ld HL, GATO_X_NEXT      ; En caso contrario, puede avanzar
      inc (HL)                ; GATO_X++
      jr Interrupcion2_Fin
Interrupcion2_MueveIzquierda:
      cp DIR_LEFT       ; Moviendose a la izquierda?
      jr NZ, Interrupcion2_MueveArriba
      ; Comprueba si el gato puede moverse hacia la izquierda
      ld A, (GATO_X)
      dec A
      cp MAX_COLUMNAS
      jr NC, Interrupcion2_Stop      
      ld D, A
      ld A, (GATO_Y)
      ld E, A
      call BloqueXY           ; En A el tipo de bloque en (D,E)
      cp 1                    ; Bloque solido, no puede avanzar
      jr Z, Interrupcion2_Stop      ; Paramos al gato
      ld HL, GATO_X_NEXT     ; En caso contrario, puede avanzar
      dec (HL)                ; GATO_X--
      jr Interrupcion2_Fin
Interrupcion2_MueveArriba:
      cp DIR_UP         ; Moviendose arriba?
      jr NZ, Interrupcion2_MueveAbajo
      ; Comprueba si el gato puede moverse hacia arriba
      ld A, (GATO_X)
      ld D, A
      ld A, (GATO_Y)
      dec A
      cp MAX_FILAS
      jr NC, Interrupcion2_Stop            
      ld E, A
      call BloqueXY           ; En A el tipo de bloque en (D,E)
      cp 1                    ; Bloque solido, no puede avanzar
      jr Z, Interrupcion2_Stop      ; Paramos al gato
      ld HL, GATO_Y_NEXT      ; En caso contrario, puede avanzar
      dec (HL)                ; GATO_Y--
      jr Interrupcion2_Fin
Interrupcion2_MueveAbajo:
      ; Comprueba si el gato puede moverse hacia abajo
      ld A, (GATO_X)
      ld D, A
      ld A, (GATO_Y)
      inc A
      cp MAX_FILAS
      jr NC, Interrupcion2_Stop      
      ld E, A
      call BloqueXY           ; En A el tipo de bloque en (D,E)
      cp 1                    ; Bloque solido, no puede avanzar
      jr Z, Interrupcion2_Stop      ; Paramos al gato
      ld HL, GATO_Y_NEXT      ; En caso contrario, puede avanzar
      inc (HL)                ; GATO_Y++
      jr Interrupcion2_Fin
Interrupcion2_Stop:     ; El gato no puede moverse mas
      ld A, DIR_STOP
      ld (GATO_MUEVE_NEXT), A
Interrupcion2_Fin:
      ; Marcamos como ocupada la casilla por donde se 
      ; esta moviendo el gato, para que asi
      ; pueda chocar contra si mismo
      ld DE, (GATO_Y_NEXT)        ; GATO_Y en E, GATO_X en D
      ld A, 1
      call PonBloqueXY

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti

; ***********************************************************
; Interrupcion 3: 
; ***********************************************************

Interrupcion3:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion4
      ld (#39), BC

      ; ld A, COLOR_BRIGHT_GREEN
      ; call ColorBorde



Interrupcion3_Fin:

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti

; ***********************************************************
; Interrupcion 4: 
; ***********************************************************

Interrupcion4:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion5
      ld (#39), BC
      
      ; ld A, COLOR_BRIGHT_BLUE
      ; call ColorBorde
     

Interrupcion4_Fin:

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti

; ***********************************************************
; Interrupcion 5: pinta el gato
; ***********************************************************

Interrupcion5:
      push HL
      push DE
      push BC
      push AF

      ; Prepara la siguiente interrupcion
      ld BC, Interrupcion0
      ld (#39), BC

      ; ld A, COLOR_BRIGHT_MAGENTA
      ; call ColorBorde

      ; Solo actualizamos cada dos frames (25Hz)
      ld A, (CONTADOR_FRAMES)
      bit 0, A
      jr Z, Interrupcion5_Fin          

      ; Pintamos el cuerpo del gato
      ld DE, (GATO_Y)        ; GATO_Y en E, GATO_X en D
      call PintaCuerpo

      ; Pintamos la cabeza del gato
      ld DE, (GATO_Y_NEXT)        ; GATO_Y_NEXT en E, GATO_X_NEXT en D
      call PintaCabeza      

      ; Actualizamos posiciones
      ld A, (GATO_X_NEXT)
      ld (GATO_X), A
      ld A, (GATO_Y_NEXT)
      ld (GATO_Y), A

      ; Guardamos la ultima direccion (si no es stop)
      ; para poder pintar correctamente los cambios
      ; de direccion en PintaCuerpo
      ld A, (GATO_MUEVE_NEXT)
      cp 0
      jr Z, Interrupcion5_Fin
      ld (GATO_MUEVE), A

Interrupcion5_Fin:

      pop AF
      pop BC
      pop DE
      pop HL
      ei
      reti