org   #100 

INCLUDE "constantes.asm"

InicioCodigo:
   di ; Deshabilita interrupciones y el firmware del CPC

   ld SP, #7F00       ; Pila 
   ld BC, #C9FB      ; 0038 EI + RET
   ld (#38), BC
 
   ; Inicializacion paleta, borde, modo de video
   ld HL, PALETA
   ld B, 16
   call ColoresPaleta

   ; Modo 0
   ld A, 0  
   call ModoPantalla

   ; Mostramos el menu principal con el gato dando vueltas
   ; y esperamos a que el jugador pulse una tecla
   call MenuPrincipal

   ; Primera pantalla 
   ld A, 0
   ld (PANTALLA_NUM), A

   ; Puntero a la primera pantalla de la lista
   ld HL, (LISTA_PANTALLAS)
   ld (PANTALLA), HL   

   ; Marcador de numero de pantalla (en BCD)
   ld A, 1
   ld (PANTALLA_NUM_BCD), A
   
IniciaJuego:

   di

   ; El color del fondo depende del numero de la pantalla
   ld A, (PANTALLA_NUM)
   call ColorFondo
   ; Borramos la pantalla con el color del fondo
   ld A, #03     
   call LimpiaPantalla

   ; Pinta la pantalla actual y la copia en el buffer
   ld DE, (PANTALLA)
   call PintaPantalla
   ld HL, (PANTALLA)
   call CopiaPantalla

   ; Pinta el indicador del numero de pantalla
   call PintaMarcador

   ; Posicion inicial del gato en esta pantalla
   ; (ultimos dos bytes de la definicion de pantalla)
   ld HL, (PANTALLA)
   ld BC, MAX_COLUMNAS*MAX_FILAS
   add HL, BC
   ld A, (HL)
   ld (GATO_X), A
   ld (GATO_X_NEXT), A
   inc HL
   ld A, (HL)
   ld (GATO_Y), A
   ld (GATO_Y_NEXT), A

   ; Resetea movimiento
   ld A, DIR_STOP
   ld (GATO_MUEVE), A
   ld (GATO_MUEVE_NEXT), A

   ; Resetea flag de game over
   ld A, 0
   ld (GAME_OVER), A
   
   ; Instalamos nuestra rutina de servicio de interrupciones
   ; justo después del VSync
   di 
   call VSync
   ld A, #C3         ; JP
   ld (#38), A       ; 0038 JP
   ld HL, Interrupcion0      
   ld (#39), HL      ; 0038 JP Int0
   ei    ; Restaura interrupciones

Loop: 
   ; Reinicia pantalla?
   ld A, (GAME_OVER)    ; Reiniciar pantalla
   cp 1
   jr Z, IniciaJuego
   cp 2
   jr Z, Loop_AvanzaPantalla
   jr Loop

Loop_AvanzaPantalla:
   ; Hemos terminado el juego?
   ld A, (PANTALLA_NUM)
   cp ULTIMA_PANTALLA   
   jp Z, JuegoTerminado    ; Si!!!

   ; Incrementa el marcador
   ld A, (PANTALLA_NUM_BCD)
   add A, 1
   daa
   ld (PANTALLA_NUM_BCD), A

   ; Incrementa el numero de pantalla
   ld A, (PANTALLA_NUM)
   inc A
   ld (PANTALLA_NUM), A

   ; Carga la siguiente pantalla de la lista
   ld HL, LISTA_PANTALLAS
   ld B, 0
   ld C, A
   add HL, BC  ; Sumamos dos veces porque son punteros (word)
   add HL, BC
   ld E, (HL)  ; Ojo, little endian
   inc HL
   ld D, (HL)
   ld (PANTALLA), DE

   ; Resetear flag de GAME OVER
   ld A, 0
   ld (GAME_OVER), A

   jp IniciaJuego


INCLUDE "menu.asm"
INCLUDE "graficos.asm"
INCLUDE "texto.asm"
INCLUDE "teclado.asm"
INCLUDE "pantalla.asm"
INCLUDE "marcador.asm"
INCLUDE "interrupciones.asm"
INCLUDE "variables.asm"
INCLUDE "gTiles.asm"
INCLUDE "gGato.asm"
INCLUDE "gNumeros.asm"
INCLUDE "pantallas.asm"
