; Menu principal con la animacion del gato y creditos
MenuPrincipal:

   ; Vamos leyendo las posiciones guardadas en ANIM_MENU_X y ANIM_MENU_Y
   ; para ir moviendo el gato alrededor de la pantalla.

   ld B, 0     ; C = Indice a la primera posicion de los array 
   ld C, 0
   push BC

   ; Empezamos pintando el cuerpo del gato (ANIM_MENU_ESTADO=1)
   ld A, 1
   ld (ANIM_MENU_ESTADO), A

   ; Reiniciamos el contador de ciclos que sirve para ir
   ; mostrando los distintos textos
   ld A, 0
   ld (ANIM_MENU_CICLOS), A

   ; Borramos la pantalla con el color del fondo
   ld A, #03     
   call LimpiaPantalla

   ; Pintamos los textos
   call MenuPrincipalTextos

MenuPrincipal_Bucle: 

   ; Cambiamos a la vez el color del fondo y del borde
   ; Usamos el contador ANIM_MENU_CICLOS para ir cambiando
   ; los colores con cada vuelta del gato
   ld A, (ANIM_MENU_CICLOS)  
   call ColorFondo

   call VSync

   pop BC      ; Recupera indice a la lista de posiciones

   ; Primero borramos la anterior cabeza o pintamos cuerpo
   ; dependiendo del valor de ANIM_MENU_ESTADO
   ld HL, ANIM_MENU_X ; Puntero al array de posiciones X
   add HL, BC
   ld D, (HL)     ; Posicion X en D
   ld HL, ANIM_MENU_Y
   add HL, BC
   ld E, (HL)     ; Posicion Y en E
   push BC        ; Guardar coordenadas
   ld A, (ANIM_MENU_ESTADO)   ; Pinta (1) o borra (0)?
   cp 1           
   jr Z, MenuPrincipal_PintaCuerpo
   ld BC, Gato_09  ; Borramos cuerpo
   jr MenuPrincipal_PintaBloque

MenuPrincipal_PintaCuerpo:
   ; Sacamos el sprite del cuerpo a pintar del array ANIM_MENU_CUERPO
   ld HL, ANIM_MENU_CUERPO
   add HL, BC
   ld C, (HL)
   ld B, 0
   ld HL, Gatotileset
   add HL, BC
   add HL, BC     ; x2 porque son punteros (2 bytes)
   ld C, (HL)
   inc HL
   ld B, (HL)
MenuPrincipal_PintaBloque:
   call PintaBloque

   ; Incrementamos la posicion para pintar la cabeza
   pop BC         ; Recuperamos la posicion donde hemos borrado o pintado el cuerpo
   inc C          ; Siguiente posicion en los arrays ANIM_MENU_X/Y
   ld A, C

   ; Si ya hemos hecho todo el recorrido, reseteamos las coordenadas BC
   cp ANIM_MENU_FRAMES 
   jr NZ, MenuPrincipal_PintaGato
   ld C, 0
   ; Alternamos entre pintar cuerpo (1) o borrar cuerpo (0)
   ld A, (ANIM_MENU_ESTADO)
   xor 1                ; 0 -> 1, 1 -> 0
   ld (ANIM_MENU_ESTADO), A 
   ; Incrementamos el contador de ciclos de animacion
   ld A, (ANIM_MENU_CICLOS)
   inc A
   ld (ANIM_MENU_CICLOS), A  
   ; Pintamos los nuevos textos para este ciclo
   call MenuPrincipalTextos

MenuPrincipal_PintaGato:
   ld HL, ANIM_MENU_X ; Puntero al array de posiciones X
   add HL, BC
   ld D, (HL)     ; Posicion X en D
   ld HL, ANIM_MENU_Y
   add HL, BC
   ld E, (HL)     ; Posicion Y en E
   push BC
   ld BC, Gato_00
   call PintaBloque

   ; Esperamos la pulsacion de una tecla para salir 
   ld B, 0
   ld C, 255   ; Numero de veces que comprueba el teclado
   call EsperaTecla
   cp 0        ; Ninguna tecla pulsada
   jp Z, MenuPrincipal_Bucle

   pop BC
   ret


; Muestra los textos del menu principal
MenuPrincipalTextos:

   push BC
   push DE
   push HL

   ; LONGATO siempre fijo
   ld HL, #C230 + 26
   ld BC, TEXTO_MENU
   call PintaTexto

   ; Dependiendo del numero de ciclos vamos
   ; mostrando una de las tres opciones
   ld A, (ANIM_MENU_CICLOS)
   and %00000011
   cp 1
   jr Z, MenuPrincipalTexto_IdeaOriginal
   cp 2 
   jr Z, MenuPrincipalTexto_VersionCPC

   ; "Pulsa una tecla"
   ld HL, #C3C0 + 8
   ld BC, TEXTO_MENU_PULSA1
   call PintaTexto

   ld HL, #C500 + 10
   ld BC, TEXTO_MENU_PULSA2
   call PintaTexto
   jr MenuPrincipalTexto_Fin

MenuPrincipalTexto_IdeaOriginal:
   ; "Idea original Martin Magni"
   ld HL, #C3C0 + 8
   ld BC, TEXTO_MENU_IDEA1
   call PintaTexto

   ld HL, #C500 + 8
   ld BC, TEXTO_MENU_IDEA2
   call PintaTexto
   jr MenuPrincipalTexto_Fin

MenuPrincipalTexto_VersionCPC:
   ;"Version CPC RetroParla"
   ld HL, #C3C0 + 8
   ld BC, TEXTO_MENU_CPC1
   call PintaTexto

   ld HL, #C500 + 8
   ld BC, TEXTO_MENU_CPC2
   call PintaTexto

MenuPrincipalTexto_Fin:
   pop HL
   pop DE
   pop BC

   ret


; Al pulsar ESC se pregunta al jugador si realmente
; quiere volver al menu principal.
ReinciarJuego:
   di    ; Paramos el bucle de interrupciones
   ; Antes de mostrar el texto de confirmacion,
   ; guardamos el fondo que va a sobreescribir para
   ; luego poder restaurarlo en caso de que el 
   ; jugador elija no salir
   ld HL, #C230
   ld A, 16
   call GuardaFondoPantalla

   ; Mostramos el texto
   ld HL, #C230 + 8
   ld BC, TEXTO_SALIR_JUEGO
   call PintaTexto

ReiniciaJuego_EsperaESC:
   ; Esperamos a que se suelte la tecla ESC
   ld B, 0
   ld C, 1   ; Numero de veces que comprueba el teclado
   call EsperaTecla
   cp 0        ; Ninguna tecla pulsada
   jp NZ, ReiniciaJuego_EsperaESC

ReiniciaJuego_EsperaRespuesta:
   ; Esperamos a que el jugador elija Si o No
   ; Leemos la fila 47 del teclado (X, C, D, S, W, E, 3, 4)
   ld A, #47
   call LeerFilaTeclado
   bit 4, A    ; S pulsada?
   jp NZ, ReiniciaJuego_CompruebaNo    ; S no pulsada, comprobamos la N
   jp InicioCodigo   ; Reinciamos el juego
ReiniciaJuego_CompruebaNo:
   ; Leemos la fila 45 del teclado (Espacio, N, J, H, Y, U, 7, 8)
   ld A, #45
   call LeerFilaTeclado
   bit 6, A    ; N pulsada?
   jp NZ, ReiniciaJuego_EsperaRespuesta    ; N no pulsada, volvemos a leer

   ; Restauramos el fondo borrado por el texto
   ld HL, #C230
   ld A, 16
   call RestauraFondoPantalla

   ; Continuamos el juego, esperamos al retrazo vertical 
   ; y volvemos a habilitar las interrupciones
   call VSync
   ld A, #C3         ; JP
   ld (#38), A       ; 0038 JP
   ld HL, Interrupcion0      
   ld (#39), HL      ; 0038 JP Int0
   ei    ; Restaura interrupciones
   jp Loop     ; Vuelve al bucle principal


; Hemos pasado la ultima pantalla! Mostramos el mensaje de
; final de juego y volvemos al menu principal
JuegoTerminado:
   di    ; Paramos el bucle de interrupciones

   ld A, 3
   call ColorFondo
   ; Borramos la pantalla con el color del fondo
   ld A, #03     
   call LimpiaPantalla

   ; "¡FELICIDADES!"
   ld HL, #C230 + 8
   ld BC, TEXTO_FIN_JUEGO1
   call PintaTexto

   ; "¡LO HAS HECHO"
   ld HL, #C3C0 + 8
   ld BC, TEXTO_FIN_JUEGO2
   call PintaTexto

   ; "MIAUUU BIEN!"
   ld HL, #C500 + 8
   ld BC, TEXTO_FIN_JUEGO3
   call PintaTexto

JuegoTerminado_Bucle:
   ; Si hay alguna tecla pulsada, esperamos que se despulse
   ; (por si el jugador se ha dejado pulsada la tecla
   ; del ultimo movimiento y se queda sin ver el final!)

   ld B, 0
   ld C, 1   ; Numero de veces que comprueba el teclado
   call EsperaTecla
   cp 0        ; Ninguna tecla pulsada
   jp NZ, JuegoTerminado_Bucle

   ; Ahora si, esperamos la pulsacion de una tecla para salir 
   ld B, 0
   ld C, 255   ; Numero de veces que comprueba el teclado
   call EsperaTecla
   cp 0        ; Ninguna tecla pulsada
   jp Z, JuegoTerminado_Bucle

   jp InicioCodigo   ; Reinciamos el juego
