; ***********************************************************
; CopiaPantalla: copia la definicion original de una pantalla
; apuntada por HL en el buffer PANTALLA_BUFFER donde se va 
; a ir modificando segun avanza el gato, marcando los tiles
; como ocupados. No podemos modificar la definicion original
; porque en caso de reiniciar el nivel tendriamos la pantalla
; machacada con los cambios de la anterior partida.
; HL: puntero a la definicion original de la pantalla
; ***************************************************a********
CopiaPantalla:
      ld DE, PANTALLA_BUFFER  ; Destino de la copia
      ld B, 0
      ld C, MAX_COLUMNAS*MAX_FILAS ; Numero de bytes a copiar
      ldir
      ret


; ***********************************************************
; BloqueXY: devuelve en A el tile de la pantalla actual 
; que esta en la posicion dada por las coordenadas(D,E)
; DE: coordenadas X,Y del tile
; ***********************************************************
BloqueXY:
      ld HL, PANTALLA_BUFFER   ; Puntero a la pantalla actual
      ; Para llegar al tile pedido, HL = HL + E*MAX_COLUMNAS + D 
      ld C, D     ; Guardamos D
      ld B, MAX_COLUMNAS
      ld D, 0
BloqueXY_Bucle:      
      add HL, DE     ; HL = HL + E*MAX_COLUMNAS
      djnz BloqueXY_Bucle
      ld E, C
      add HL, DE     ; HL = HL + D 
      ld A, (HL)
      ret

; ***********************************************************
; PonBloqueXY: modifica el valor de la casilla (D,E) 
; DE: coordenadas X,Y del tile
; A: tipo de tile 
; ***********************************************************
PonBloqueXY:
      ld HL, PANTALLA_BUFFER    ; Puntero a la pantalla actual
      ; Para llegar al tile pedido, HL = HL + E*MAX_COLUMNAS + D 
      ld C, D     ; Guardamos D
      ld B, MAX_COLUMNAS
      ld D, 0
PonBloqueXY_Bucle:      
      add HL, DE     ; HL = HL + E*MAX_COLUMNAS
      djnz PonBloqueXY_Bucle
      ld E, C
      add HL, DE     ; HL = HL + D 
      ld (HL), A
      ret

; ***********************************************************
; Bloqueado: comprueba si hay algun movimiento posible (A = 0)
; o si el gato esta bloqueado (A = 1) a partir de la posicion
; actual (GATO_X, GATO_Y)
; ***********************************************************
Bloqueado:
      ; Salimos en cuanto encontremos un movimiento posible
      ; Puede moverse hacia la derecha?
      ; Comprobar primero si esta en el borde de la pantalla
      ld DE, (GATO_Y)         ; GATO_Y en E, GATO_X en D
      ld A, D
      inc A
      cp MAX_COLUMNAS
      jr NC, Bloqueado_CompruebaIzquierda ; Gato en borde derecho
      ; Comprobar si hay bloque solido a la derecha
      inc D
      call BloqueXY     ; En A el tipo de tile (1 = solido)
      cp 1
      jr Z, Bloqueado_CompruebaIzquierda
      ; Hay hueco a la derecha, salimos
      ld A, 0
      ret
Bloqueado_CompruebaIzquierda:
      ; Puede moverse hacia la izquierda?
      ; Comprobar primero si esta en el borde de la pantalla
      ld DE, (GATO_Y)         ; GATO_Y en E, GATO_X en D
      ld A, D
      dec A
      cp MAX_COLUMNAS
      jr NC, Bloqueado_CompruebaArriba ; Gato en borde izquierdo
      ; Comprobar si hay bloque solido a la izquierda
      dec D
      call BloqueXY     ; En A el tipo de tile (1 = solido)
      cp 1
      jr Z, Bloqueado_CompruebaArriba
      ; Hay hueco a la izquierda, salimos
      ld A, 0
      ret
Bloqueado_CompruebaArriba:
      ; Puede moverse hacia ariba?
      ; Comprobar primero si esta en el borde de la pantalla
      ld DE, (GATO_Y)         ; GATO_Y en E, GATO_X en D
      ld A, E
      dec A
      cp MAX_FILAS
      jr NC, Bloqueado_CompruebaAbajo ; Gato en borde superior
      ; Comprobar si hay bloque solido arriba
      dec E
      call BloqueXY     ; En A el tipo de tile (1 = solido)
      cp 1
      jr Z, Bloqueado_CompruebaAbajo
      ; Hay hueco hacia arriba, salimos
      ld A, 0
      ret
Bloqueado_CompruebaAbajo:
      ; Puede moverse hacia abajo?
      ; Comprobar primero si esta en el borde de la pantalla
      ld DE, (GATO_Y)         ; GATO_Y en E, GATO_X en D
      ld A, E
      inc A
      cp MAX_FILAS
      jr NC, Bloqueado_Fin ; Gato en borde inferior
      ; Comprobar si hay bloque solido abajo
      inc E
      call BloqueXY     ; En A el tipo de tile (1 = solido)
      cp 1
      jr Z, Bloqueado_Fin
      ; Hay hueco hacia abajo, salimos
      ld A, 0
      ret
Bloqueado_Fin:
      ld A, 1     ; No hay movimientos posibles, gato bloqueado
      ret

; ***********************************************************
; PantallaCompletada: recorre los tiles de la pantalla
; para comprobar si queda alguno sin cubrir.
; A = 1 si pantalla completada, A != 1 si quedan huecos
; ***********************************************************
PantallaCompletada:
      ld HL, PANTALLA_BUFFER       ; Puntero a la pantalla actual
      ; Recorremos las MAX_FILAS*MAX_COLUMNAS casillas.
      ; En cuanto encontremos una vacia, salimos con A != 1
      ld B, MAX_FILAS*MAX_COLUMNAS
PantallaCompletada_Bucle:
      ld A, (HL)
      cp 1
      ret NZ      ; Una casilla vacia, salimos con A != 1
      inc HL
      djnz PantallaCompletada_Bucle
      ret         ; Todas las casillas ocupadas, salimos con A = 1
