; LeerTeclado: rellena el array TECLADO con el estado completo de las teclas
RellenaMapaTeclado:
   ;;;;;; di
   ld HL, MAPA_TECLADO
   ld BC, #F782
   out (C), C
   ld BC, #F40E
   ld E, B
   out (C), C
   ld BC, #F6C0
   ld D, B
   out (C),C
   ld C, 0
   out (C), C
   ld BC, #F792
   out (C), C
   ld A, #40
   ld C, #4a
RellenaMapaTeclado_Bucle:
   ld B, D
   out (C), A
   ld B, E
   ini
   inc A
   cp C
   jr C, RellenaMapaTeclado_Bucle
   ld BC, #F782
   out (C), C
   ;;;;;; ei
   ret

; Chequea unicamente la pulsacion de las teclas del juego
LeerTeclado:
   call RellenaMapaTeclado
   ld HL, MAPA_TECLADO     ; Inicio del array de teclas
   ld DE, 0                ; Reseteamos bits de pulsaciones
   ; Fila 0: INTRO, DOWN, RIGHT, UP
   ld A, (HL)
LeerTeclado_INTRO:
   bit KEY_INTRO_MASK, A
   jp NZ, LeerTeclado_DOWN
   set KEY_RETURN, D
LeerTeclado_DOWN:
   bit KEY_DOWN_MASK, A
   jp NZ, LeerTeclado_UP
   set KEY_DOWN, D
LeerTeclado_UP:
   bit KEY_UP_MASK, A
   jp NZ, LeerTeclado_RIGHT
   set KEY_UP, D
LeerTeclado_RIGHT:
   bit KEY_RIGHT_MASK, A
   jp NZ, LeerTeclado_LEFT   
   set KEY_RIGHT, D
LeerTeclado_LEFT:
   inc HL      ; Fila 1: LEFT
   ld A, (HL)
   bit KEY_LEFT_MASK, A
   jp NZ, LeerTeclado_SPACE
   set KEY_LEFT, D
LeerTeclado_SPACE:
   inc HL      ; Fila 2: RETURN
   inc HL      ; Fila 3: P
   inc HL      ; Fila 4: O
   inc HL      ; Fila 5: SPACE
   ld A, (HL)
   bit KEY_SPACE_MASK, A
   jp NZ, LeerTeclado_ESC
   set KEY_FIRE, D
LeerTeclado_ESC:
   inc HL      ; Fila 6
   inc HL      ; Fila 7
   inc HL      ; Fila 8: ESC
   ld A, (HL)
   bit KEY_ESC_MASK, A
   jp NZ, LeerTeclado_JOY_FIRE
   set KEY_ESCAPE, D   
LeerTeclado_JOY_FIRE:
   inc HL      ; Fila 9: JOYSTICK, JUGADOR 2
   ld A, (HL)
   bit KEY_JOY_FIRE_MASK, A
   jp NZ, LeerTeclado_JOY_DOWN
   set KEY_FIRE, D
LeerTeclado_JOY_DOWN:
   bit KEY_JOY_DOWN_MASK, A
   jp NZ, LeerTeclado_JOY_UP
   set KEY_DOWN, D
LeerTeclado_JOY_UP:
   bit KEY_JOY_UP_MASK, A
   jp NZ, LeerTeclado_JOY_RIGHT
   set KEY_UP, D
LeerTeclado_JOY_RIGHT:
   bit KEY_JOY_RIGHT_MASK, A
   jp NZ, LeerTeclado_JOY_LEFT
   set KEY_RIGHT, D
LeerTeclado_JOY_LEFT:
   bit KEY_JOY_LEFT_MASK, A
   jp NZ, LeerTeclado_Fin
   set KEY_LEFT, D
LeerTeclado_Fin:
   ld HL, TECLAS_PULSADAS
   ld (HL), D   ; Guarda estado de teclas en RAM
   ret


; Comprueba si se ha pulsado una tecla.
; El teclado se lee tantas veces como se especique
; en el registro BC y luego sale de la rutina.
; Si BC es cero, el teclado se lee indefinidamente (TO DO)
; Salida: A = 0 si no se ha pulsado ninguna tecla.
EsperaTecla:
   push BC
   call RellenaMapaTeclado
   ld HL, MAPA_TECLADO     ; Inicio del array de teclas
   ld B, 10     ; 10 lineas de teclado
EsperaTecla_CompruebaFila:
   ld A, (HL)
   cp #FF      ; FF = ninguna tecla pulsada en esa fila
   jp NZ, EsperaTecla_Fin
   inc HL
   djnz EsperaTecla_CompruebaFila
   pop BC
   dec BC
   ld A, B  ; BC = 0?
   cp 0
   jp NZ, EsperaTecla
   ld A, C
   cp 0
   jp NZ, EsperaTecla
   ret   ; Tecla no pulsada, A = 0
EsperaTecla_Fin:
   pop BC   ; Restaura la pila y sale con A != 0
   ret


; Lee una unica fila de teclado.
; Entrada: A = fila a leer (#40 a #49)
; Salida: A = bits de las teclas de esa fila
; 0 no pulsada, 1 pulsada
LeerFilaTeclado:
   ld D,0
   ld BC, #F782 ; PPI port A out /C out 
   out (C), C 
   ld BC, #F40E ; Select Ay reg 14 on ppi port A 
   out (C), C 
   ld BC, #F6C0 ; This value is an AY index (R14) 
   out (C), C 
   out (C), D ; Validate!! out (c),0
   ld BC, #F792 ; PPI port A in/C out 
   out (C), C 
   dec B 
   out (C), A ; Send KbdLine on reg 14 AY through ppi port A
   ld B, #F4 ; Read ppi port A 
   in A, (C) ; e.g. AY R14 (AY port A) 
   ld BC, #F782 ; PPI port A out / C out 
   out (C), C 
   dec B ; Reset PPI Write 
   out (C), D ; out (c),0
   ret